'use strict';

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const { Clutter, Gio, GLib, GObject, St } = imports.gi;
const { main, panelMenu } = imports.ui;

const LatencyIndicator = GObject.registerClass(
	class LatencyIndicator extends panelMenu.Button {
		_init() {
			super._init(0, `LatencyIndicator`, false);

			this.icon = new St.Icon({
				gicon: new Gio.ThemedIcon({ name: 'network-transmit-receive' }),
				style_class: 'system-status-icon'
			});

			this.label = new St.Label({
				y_expand: true,
				y_align: Clutter.ActorAlign.CENTER
			});
			this._update();

			this.box = new St.BoxLayout();
			this.box.add(this.icon);
			this.box.add(this.label);

			this.add_child(this.box);
		}

		_update() {
			execCommand(["ping", "-c", "2", "1.1.1.1"]).then(stdout => {
				let latency = stdout;
				latency = latency.split(/\n/);
				latency = latency[latency.length - 2].toString(); // rtt min/avg/max/mdev = 13.109/21.812/38.923/9.078 ms
				latency = latency.split(/\s/)[3].toString();      // 13.109/21.812/38.923/9.078
				latency = latency.split(/\//)[1].toString();      // 21.812
				latency = Math.round(latency).toString();         // 21
				this.label.text = `${latency}ms`;
			});

			this._timeout = GLib.timeout_add_seconds(GLib.PRIORITY_DEFAULT, 5, this._update.bind(this));
			return GLib.SOURCE_REMOVE;
		}

		_onDestroy() {
			if (this._timeout) GLib.source_remove(this._timeout);

			super._onDestroy();
		}
	}
)


let latencyIndicator;

function init() {
	log(`Initializing ${Me.metadata.name} version ${Me.metadata.version}`);
}

function enable() {
	latencyIndicator = new LatencyIndicator();
	main.panel.addToStatusArea('latencyIndicator', latencyIndicator);
}

function disable() {
	latencyIndicator.destroy();
}


/**
 * execCommand:
 * @argv: an array of arguments
 * @input: (nullable): input text
 * @cancellable: (nullable): an optional Gio.Cancellable to cancel the execution
 *
 * A simple, asynchronous process launcher wrapped in a Promise.
 *
 * Returns: stdout text output
 */
async function execCommand(argv, input = null, cancellable = null) {
	try {
		// We'll assume we want output, or that returning none is not a problem
		let flags = Gio.SubprocessFlags.STDOUT_PIPE;

		// If we aren't given any input, we don't need to open stdin
		if (input !== null)
			flags |= Gio.SubprocessFlags.STDIN_PIPE;

		let proc = new Gio.Subprocess({
			argv: argv,
			flags: flags
		});

		// Classes that implement GInitable must be initialized before use, but
		// an alternative in GJS is to just use Gio.Subprocess.new(argv, flags)
		proc.init(cancellable);

		let stdout = await new Promise((resolve, reject) => {
			proc.communicate_utf8_async(input, cancellable, (proc, res) => {
				try {
					let [ok, stdout, stderr] = proc.communicate_utf8_finish(res);
					resolve(stdout);
				} catch (e) {
					reject(e);
				}
			});
		});

		return stdout;
	} catch (e) {
		logError(e);
	}
}
